// SPDX-License-Identifier: MIT
/*
 * Copyright (c) 2023 Atmark Techno,Inc.
 */

#include <stdint.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "device-info.h"

#define IMX_OCOTP_MVMEM_IMX8MP_PRODUCT   0xE0
#define IMX_OCOTP_MVMEM_IMX6ULL_PRODUCT  0xA0
#define IMX_OCOTP_MVMEM_PRODUCT_SIZE     2

#define IMX_OCOTP_MVMEM_IMX8MP_LOT_SN    0xE4
#define IMX_OCOTP_MVMEM_IMX6ULL_LOT_SN   0xA4
#define IMX_OCOTP_MVMEM_LOT_SN_SIZE      4

#define IMX_OCOTP_MVMEM_IMX8MP_MAC_ADDR  0x90
#define IMX_OCOTP_MVMEM_IMX6ULL_MAC_ADDR 0x88
#define IMX_OCOTP_MVMEM_MAC_ADDR_SIZE    6

enum board_type {
	A640_ES_PRODUCT_ID = 0x0097,
	A640_PRODUCT_ID = 0x009C,
	A610_ES_PRODUCT_ID = 0x00B4,
	A610_PRODUCT_ID = 0x00B7,
	G4_EVA_PRODUCT_ID = 0x00C6,
	G4_ES1_PRODUCT_ID = 0x00C7,
	G4_ES2_PRODUCT_ID = 0x00C8,
	G4_PRODUCT_ID = 0x00C9,
	G4_ES3_PRODUCT_ID = 0x00CB,
	A6E_PRODUCT_ID = 0x00CE,
	X2_ES1_PRODUCT_ID = 0x00D1,
	X2_ES2_PRODUCT_ID = 0x00D2,
	X2_PRODUCT_ID = 0x00D3,
	BOARD_TYPE_UNKNOWN,
};

#define OCOTP_MAX_OFFSET 0x100
struct armadillo {
	enum soc soc;
	enum board_type board_type;
	int mac_addr_num;
	const char *product_name;
	unsigned char ocotp_data[OCOTP_MAX_OFFSET];
};

static struct armadillo armadillo;

static enum board_type get_board_type(void)
{
	uint16_t board_type;

	switch (armadillo.soc) {
	case IMX8MP:
		memcpy(&board_type, armadillo.ocotp_data + IMX_OCOTP_MVMEM_IMX8MP_PRODUCT,
			IMX_OCOTP_MVMEM_PRODUCT_SIZE);
		break;
	case IMX6ULL:
		memcpy(&board_type, armadillo.ocotp_data + IMX_OCOTP_MVMEM_IMX6ULL_PRODUCT,
			IMX_OCOTP_MVMEM_PRODUCT_SIZE);
		break;
	default:
		return BOARD_TYPE_UNKNOWN;
	}

	return board_type;
}


static int init_armadillo(void)
{
	armadillo.soc = get_soc();
	if (armadillo.soc == SOC_UNKNOWN) {
		return 1;
	}

	if (imx_ocotp_read(&armadillo.ocotp_data, OCOTP_MAX_OFFSET) < 0) {
		return 1;
	}

	armadillo.board_type = get_board_type();
	armadillo.mac_addr_num = 1;
	switch (armadillo.board_type) {
	case A640_ES_PRODUCT_ID:
	case A640_PRODUCT_ID:
		armadillo.product_name = "Armadillo-640";
		break;
	case A610_ES_PRODUCT_ID:
	case A610_PRODUCT_ID:
		armadillo.product_name = "Armadillo-610";
		break;
	case G4_EVA_PRODUCT_ID:
	case G4_ES1_PRODUCT_ID:
	case G4_ES2_PRODUCT_ID:
	case G4_PRODUCT_ID:
	case G4_ES3_PRODUCT_ID:
		armadillo.product_name = "Armadillo-IoT G4";
		armadillo.mac_addr_num = 2;
		break;
	case A6E_PRODUCT_ID:
		armadillo.product_name = "Armadillo-IoT A6E";
		break;
	case X2_ES1_PRODUCT_ID:
	case X2_ES2_PRODUCT_ID:
	case X2_PRODUCT_ID:
		armadillo.product_name = "Armadillo-X2";
		break;
	default:
		fprintf(stderr, "unsupported products: %X\n", armadillo.board_type);
		return 1;
	}

	return 0;
}

int print_product_name(void)
{
	int rc;

	if (armadillo.soc == SOC_UNKNOWN && init_armadillo() < 0)
		return 1;

	if (print_env)
		rc = printf("export AT_PRODUCT_NAME='%s'\n", armadillo.product_name);
	else
		rc = printf("%s\n", armadillo.product_name);

	return rc < 0;
}

int print_serial_num(void)
{
	int rc;
	unsigned char *buf;

	if (armadillo.soc == SOC_UNKNOWN && init_armadillo() < 0)
		return 1;

	switch (armadillo.soc) {
	case IMX6ULL:
		buf = armadillo.ocotp_data + IMX_OCOTP_MVMEM_IMX6ULL_LOT_SN;
		break;
	case IMX8MP:
		buf = armadillo.ocotp_data + IMX_OCOTP_MVMEM_IMX8MP_LOT_SN;
		break;
	default:
		// should never happen after init_armadillo() success...
		xfail("Invalid soc after init_armadillo() success");
	}

	if (print_env)
		rc = printf("export AT_SERIAL_NUMBER=%04X%02X%02X%02X%02X\n",
			    armadillo.board_type, buf[3], buf[2], buf[1], buf[0]);
	else
		rc = printf("%04X%02X%02X%02X%02X\n",
			    armadillo.board_type, buf[3], buf[2], buf[1], buf[0]);

	return rc < 0;
}

int print_mac_addr(void)
{
	int i, offset = 0;
	int rc = 0;


	if (armadillo.soc == SOC_UNKNOWN && init_armadillo() < 0)
		return 1;

	switch (armadillo.soc) {
	case IMX6ULL:
		offset = IMX_OCOTP_MVMEM_IMX6ULL_MAC_ADDR;
		break;
	case IMX8MP:
		offset = IMX_OCOTP_MVMEM_IMX8MP_MAC_ADDR;
		break;
	default:
		// should never happen after init_armadillo() success...
		xfail("Invalid soc after init_armadillo() success");
	}

	for (i = 0; i < armadillo.mac_addr_num; i++) {
		unsigned char *buf = armadillo.ocotp_data + offset + i * 6;

		if (print_env)
			rc = printf("export AT_LAN_MAC%d=%02X:%02X:%02X:%02X:%02X:%02X\n",
				    i+1, buf[5], buf[4], buf[3], buf[2], buf[1], buf[0]);
		else
			rc = printf("eth%d: %02X:%02X:%02X:%02X:%02X:%02X\n",
				    i, buf[5], buf[4], buf[3], buf[2], buf[1], buf[0]);

		if (rc < 0)
			break;
	}

	return rc < 0;
}
